@@ -5,22 +5,28 @@
 #
 ####################################
 
-estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300, x.step.pi=300, nb.rei=100, method=3, gridsize=300){
-  
+
+estim.mix=function(tab, tab.imp, conditions, x.step.mod=300, x.step.pi=300, nb.rei=100, method=1, gridsize=300){
+
   #require(Iso);
-  
+  min.x=min(tab.imp,na.rm=TRUE);
+  max.x=max(tab.imp,na.rm=TRUE);
+
+  x.min=min.x-(max.x-min.x)/2;
+  x.max=max.x+(max.x-min.x)/2;
+
   #Function to minimize : Weibull assumption
-  fr <- function(pi,x,pi_est,w,Ft) {   
+  fr <- function(pi,x,pi_est,w,Ft) {
     p <- pi[1];
     b <- pi[2];
     c <- pi[3];
     f=p+(1-p)*exp(-b*(x)^c)/(1-Ft);
     return(sum(w*(pi_est-f)^2))
   }
-  
+
   #Number of conditions
   nb_cond=length(levels(conditions));
-  
+
   nb_rep=rep(0,nb_cond);
   pi_abs=rep(0,nb_cond);
   pi_na=rep(0,length(tab[1,]));
@@ -36,18 +42,18 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
   pi.init.pava=matrix(0,x.step.pi,length(tab[1,]));
   FTOT.i=matrix(0,x.step.pi,length(tab[1,]));
   FNA.i=matrix(0,x.step.pi,length(tab[1,]));
-  
+
   q.n=matrix(0,100,length(tab[1,]));
   q.c=matrix(0,100,length(tab[1,]));
   s.q.n=rep(0,length(tab[1,]));
   s.q.c=rep(0,length(tab[1,]));
-    
+
   ABSC=matrix(0,x.step.mod,length(tab[1,]));
   FTOT=matrix(0,x.step.mod,length(tab[1,]));
   FNA=matrix(0,x.step.mod,length(tab[1,]));
   FOBS=matrix(0,x.step.mod,length(tab[1,]));
   FMNAR=matrix(0,x.step.mod,length(tab[1,]));
-  
+
   k=1;
   j=1;
   for (i in 1:nb_cond){
@@ -67,49 +73,51 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
     tab_imp2=as.matrix(tab.imp[liste_garde,]);
 
     ############
-    
+
     while (j<(k+nb_rep[i])){
       #Percentage of missing values in sample j
       nna=sum(is.na(tab2[,j]));
       pi_na[j]=nna/length(tab2[,j]);
-      
+
       ############
       #Rough estimation of the distribution of missing values in sample j
       v_na=tab_imp2[is.na(tab2[,j]),j];
       F_na=ecdf(v_na);
-      
+
       ############
       #Distribution of observed values in sample j
       F_obs=ecdf(tab2[which(!is.na(tab2[,j])),j]);
-      
+
       ############
       #Determination of the interval on which is estimated pi^MLE(x)
       #F_na(x)>0 et F_obs(x)>0
       xmin=max(sort(v_na,na.last=T)[1],sort(tab2[which(!is.na(tab2[,j])),j],na.last=T)[1]);
+      #print(xmin)
       #F_na(x)<1 et F_obs(x)<1
       xmax=min(-sort(-v_na,na.last=T)[2],-sort(-tab2[which(!is.na(tab2[,j])),j],na.last=T)[2]);
+      #print(xmax)
       absi[,j]=seq(xmin,xmax,length.out=x.step.pi)
       pi_init=NULL;
       for (x in absi[,j]){
-        #Initial estimator of the proportion (1-F_na(x))/(1-F(x)) 
+        #Initial estimator of the proportion (1-F_na(x))/(1-F(x))
         pi_init_x=(1-F_na(x))/(1-(pi_na[j]*F_na(x)+(1-pi_na[j])*F_obs(x)))
         pi_init=c(pi_init,pi_init_x)
       }
       kx=1
       while (pi_init[kx]>=1){
-            kx=kx+1;
+        kx=kx+1;
       }
       xmin=absi[kx,j];
-      
+
       abs[,j]=seq(xmin,xmax,length.out=x.step.pi)
-      
+
       ############
       #Estimation of pi^MLE(x)
       pi_init=NULL;
       F_tot1=NULL;
       F_na1=NULL;
       Fobs=NULL;
-      
+
       varasy=NULL;
       del=NULL;
       mm=NULL;
@@ -118,7 +126,7 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
       kk=NULL;
       for (x in abs[,j]){
         ############
-        #Initial estimator of the proportion (1-F_na(x))/(1-F(x)) 
+        #Initial estimator of the proportion (1-F_na(x))/(1-F(x))
         pi_init_x=(1-F_na(x))/(1-(pi_na[j]*F_na(x)+(1-pi_na[j])*F_obs(x)))
         pi_init=c(pi_init,pi_init_x)
 
@@ -127,13 +135,13 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
         F_na1=c(F_na1,F_na(x))
         Fobs=c(Fobs,F_obs(x))
         F_tot1=c(F_tot1,pi_na[j]*F_na(x)+(1-pi_na[j])*F_obs(x))
-        
+
         ############
         #Estimation of the asymptotic variance of the estimator
         del=(pi_na[j]-1)*pi_init_x/(pi_na[j]*pi_init_x-1)
         mm=(1-del*(1-F_obs(x)))/(pi_init_x*((1-F_obs(x))*(pi_na[j]-1)-pi_na[j])+1)^2
         a=pi_na[j]
-        p=(1-F_obs(x)) 
+        p=(1-F_obs(x))
         I22=del*p*(1-del*p)*((1/pi_init_x+p*(1-a)/(a*(p-1)*pi_init_x-p*pi_init_x+1))^2)/(1-a*pi_init_x)^2
         I11=(1/a-1)/(p*(1-p))+del*p*(1-del*p)*((1/p-pi_init_x*(1-a)/(a*(p-1)*pi_init_x-p*pi_init_x+1))^2)
         I12=(1-pi_na[j])*mm
@@ -141,25 +149,29 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
         v=solve(I)[2,2]
         varasy=c(varasy,((1-pi_na[j])/pi_na[j])*v)
       }
+
       varasy[varasy<0]=1e-10;
+      #varasy=1+(varasy-min(varasy))/(max(varasy)-min(varasy))
+      #varasy=rep(1,length(varasy));
       PI_INIT[,j]=pi_init;
       VAR_PI_INIT[,j]=varasy;
-      
+
       #Function to minimize
       #The minimization is performed nb.rei times to try to find a global minimum
       pim=matrix(0,nb.rei,4)
       for (nbit in 1:nb.rei){
         init=c(runif(1,0,0.5),runif(1,1,50),runif(1,1,10))
-        while (!inherits(try(optim(init, fr, gr=NULL, x=(abs[,j]-xmin) , pi_est=pi_init, lower=rep(0,3), upper=c(1,Inf,Inf), method="L-BFGS-B", w=1/varasy, Ft=F_tot1), TRUE), "try-error")==FALSE){
-          init=c(runif(1,0,1),runif(1,1,50),runif(1,1,10));
+        #while (!inherits(try(optim(init, fr, gr=NULL, x=(abs[,j]-xmin) , pi_est=pi_init, lower=rep(0,3), upper=c(1,Inf,Inf), method="L-BFGS-B", w=1/varasy, Ft=F_tot1), TRUE), "try-error")==FALSE){
+        while (!inherits(try(optim(init, fr, gr=NULL, x=(abs[,j]-xmin) , pi_est=pi_init, lower=c(0.01,0.0005,2), upper=c(0.5,0.01,5), method="L-BFGS-B", w=1/varasy, Ft=F_tot1), TRUE), "try-error")==FALSE){
+            init=c(runif(1,0,1),runif(1,1,50),runif(1,1,10));
         }
-        re=optim(init, fr, gr=NULL, x=(abs[,j]-xmin) , pi_est=pi_init, lower=rep(0,3), upper=c(1,Inf,Inf), method="L-BFGS-B", w=1/varasy, Ft=F_tot1);
+        re=optim(init, fr, gr=NULL, x=(abs[,j]-xmin) , pi_est=pi_init, lower=c(0.01,0.0005,2), upper=c(0.5,0.01,5), method="L-BFGS-B", w=1/varasy, Ft=F_tot1);
         pim[nbit,1]=re$par[1]
         pim[nbit,2]=re$par[2]
         pim[nbit,3]=re$par[3]
         pim[nbit,4]=re$value
       }
-      
+
       #Final estimation of the proportion of MCAR values with different methods
       pi_trend=rep(0,length(abs[,j]))
       pi_trend_pava=rep(0,length(abs[,j]))
@@ -194,8 +206,8 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
         distance=NULL
         for(kl in 1:gridsize){
           a=kl/gridsize ## Assumes a value of the mixing proportion
-          F.hat=(F_na1-(1-a)*F_tot1)/a ## Computes the naive estimator 
-          F.is=pava(F.hat,Freq,decreasing=FALSE) ## Computes the isotonic estimator 
+          F.hat=(F_na1-(1-a)*F_tot1)/a ## Computes the naive estimator
+          F.is=pava(F.hat,Freq,decreasing=FALSE) ## Computes the isotonic estimator
           F.is[which(F.is<=0)]=0
           F.is[which(F.is>=1)]=1
           distance=c(distance,a*sqrt(t((F.hat[!is.nan(F.is)]-F.is[!is.nan(F.is)])^2)%*%Freq[!is.nan(F.is)]));
@@ -224,12 +236,12 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
       gamma=pi_na[j]*(1-pi_mcar[j])/(1-pi_mcar[j]*pi_na[j]);
       upper.q=0.99
       interv=gamma+(1-gamma)*seq(sta[j],upper.q,length.out=100);
-      
+
       q.n[,j]=qnorm(gamma+(1-gamma)*seq(0.01,upper.q,length.out=100), mean = 0, sd = 1);
       q.c[,j]=quantile(tab2[,j], probs = seq(0.01,upper.q,length.out=100), na.rm = T);
       s.q.n[j]=qnorm(gamma+(1-gamma)*sta[j], mean = 0, sd = 1);
       s.q.c[j]=quantile(tab2[,j], probs = sta[j], na.rm = T);
-      
+
       q.normal = qnorm(interv, mean = 0, sd = 1);
       q.curr.sample = quantile(tab2[,j], probs = seq(sta[j],upper.q,length.out=100), na.rm = T);
       temp.QR = lm(q.curr.sample ~ q.normal);
@@ -239,7 +251,7 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
 
       ############
       #Final estimation of the mixture model on the interval precised in input
-      
+
       ABSC[,j]=seq(x.min,x.max,length.out=x.step.mod);
 
       F_tot=pnorm(ABSC[,j],mean=m,sd=sqrt(v));
@@ -248,7 +260,7 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
       F_mnar=pava(F_mna,decreasing=F);
       F_mnar[F_mnar>1]=1;
       F_mnar[F_mnar<0]=0;
-      
+
       #Reestimation of the cdfs
       kl=1
       F_na2=NULL
@@ -262,24 +274,24 @@ estim.mix=function(tab, tab.imp, conditions, x.min=20, x.max=30, x.step.mod=300,
         F_tot2=c(F_tot2,pi_na[j]*(aa+bb)+(1-pi_na[j])*F_obs(x));
         kl=kl+1;
       }
-      
+
       FTOT.i[,j]=F_tot1;
       FNA.i[,j]=F_na1;
       FTOT[,j]=F_tot2;
       FNA[,j]=F_na2;
       FOBS[,j]=Fobs;
       FMNAR[,j]=F_mnar;
-      
+
       j=j+1;
     }
-    
+
     k=k+nb_rep[i];
   }
-  
+
   return(list(abs.pi=abs,pi.init=PI_INIT,var.pi.init=VAR_PI_INIT,
+              #pi.m=pim,
               #F.na.i=FNA.i,F.tot.i=FTOT.i,pi.trend=pi.trend,pi.trend.pava=pi.trend.pava,pi.init.pava=pi.init.pava,
               abs.mod=ABSC[,1],pi.na=pi_na,F.na=FNA,F.tot=FTOT,F.obs=FOBS,F.mnar=FMNAR,pi.mcar=pi_mcar
               #,eta=sta,q.n=q.n,q.c=q.c,s.q.n=s.q.n,s.q.c=s.q.c
-              ));
+  ));
 }
-
